local BioInd = require('common')('Bio_Industries')

local ICONPATH = BioInd.modRoot .. "/graphics/icons/"
local ENTITYPATH = BioInd.modRoot .. "/graphics/entities/biofarm/"

require ("prototypes.Bio_Farm.pipeConnectors")
require ("util")


inv_extension2 = {
  filename = ENTITYPATH .. "Bio_Farm_Idle_alt.png",
  priority = "high",
  width = 320,
  height = 320,
  frame_count = 1,
  direction_count = 1,
  shift = {0.75, 0},
}


data:extend({
  ------- Seedling
  {
    type = "simple-entity-with-force",
    name = "seedling",
    localised_name = {"entity-name.seedling"},
    localised_description = {"entity-description.seedling"},
    icon = ICONPATH .. "Seedling.png",
    icon_size = 64,
    icons = {
      {
        icon = ICONPATH .. "Seedling.png",
        icon_size = 64,
      }
    },
    order = "x[bi]-a[bi-seedling]",
    flags = {"placeable-neutral", "placeable-player", "player-creation", "breaths-air"},
    create_ghost_on_death = false,
    minable = {
      mining_particle = "wooden-particle",
      mining_time = 0.25,
      result = "seedling",
      count = 1
    },
    corpse = nil,
    remains_when_mined = nil,
	  emissions_per_second = { pollution = -0.0006 },
    max_health = 5,

    collision_box = {{-0.1, -0.1}, {0.1, 0.1}},
    selection_box = {{-0.5, -0.5}, {0.5, 0.5}},
    subgroup = "trees",
    --vehicle_impact_sound = sounds.car_wood_impact,
    picture = {
      filename = ICONPATH .. "Seedling_b.png",
      priority = "extra-high",
      width = 64,
      height = 64,
      --~ scale = 0.75
      scale = 0.3
    },
  },

  ------- Seedling - Dummy for Seed Bomb
  {
    type = "simple-entity-with-force",
    name = "seedling-2",
    localised_name = {"entity-name.seedling"},
    localised_description = {"entity-description.seedling"},
    icon = ICONPATH .. "Seedling.png",
    icon_size = 64,
    icons = {
      {
        icon = ICONPATH .. "Seedling.png",
        icon_size = 64,
      }
    },
    order = "x[bi]-a[bi-seedling]",
    flags = {"placeable-neutral", "placeable-player", "player-creation", "breaths-air"},
    create_ghost_on_death = false,
    minable = {
      mining_particle = "wooden-particle",
      mining_time = 0.25,
      result = "seedling",
      count = 1
    },
    corpse = nil,
    remains_when_mined = nil,
	emissions_per_second = { pollution = -0.0006 },
    max_health = 5,

    collision_box = {{-0.1, -0.1}, {0.1, 0.1}},
    selection_box = {{-0.5, -0.5}, {0.5, 0.5}},
    subgroup = "trees",
    --vehicle_impact_sound = sounds.car_wood_impact,
    picture = {
      filename = ICONPATH .. "Seedling_b.png",
      priority = "extra-high",
      width = 64,
      height = 64,
      scale = 0.3
    },
  },

  {
    type = "simple-entity-with-force",
    name = "seedling-3",
    localised_name = {"entity-name.seedling"},
    localised_description = {"entity-description.seedling"},
    icon = ICONPATH .. "Seedling.png",
    icon_size = 64,
    icons = {
      {
        icon = ICONPATH .. "Seedling.png",
        icon_size = 64,
      }
    },
    order = "x[bi]-a[bi-seedling]",
    flags = {"placeable-neutral", "placeable-player", "player-creation", "breaths-air"},
    create_ghost_on_death = false,
    minable = {
      mining_particle = "wooden-particle",
      mining_time = 0.25,
      result = "seedling",
      count = 1
    },
    corpse = nil,
    remains_when_mined = nil,
	emissions_per_second = { pollution = -0.0006 },
    max_health = 5,

    collision_box = {{-0.1, -0.1}, {0.1, 0.1}},
    selection_box = {{-0.5, -0.5}, {0.5, 0.5}},
    subgroup = "trees",
    --vehicle_impact_sound = sounds.car_wood_impact,
    picture = {
      filename = ICONPATH .. "Seedling_b.png",
      priority = "extra-high",
      width = 64,
      height = 64,
      scale = 0.3
    },
  },


  ------- Bio Farm
  {
    type = "assembling-machine",
    name = "bi-bio-farm",
    icon = ICONPATH .. "Bio_Farm_Icon.png",
    icon_size = 64,
    icons = {
      {
        icon = ICONPATH .. "Bio_Farm_Icon.png",
        icon_size = 64,
      }
    },
    flags = {"placeable-neutral", "placeable-player", "player-creation"},
    minable = {hardness = 0.2, mining_time = 0.5, result = "bi-bio-farm"},
    max_health = 250,
    corpse = "big-remnants",
    dying_explosion = "medium-explosion",
    resistances = {{type = "fire", percent = 70}},

    fluid_boxes = {
      {
        production_type = "input",
        pipe_picture = assembler3pipepictures(),
        pipe_covers = pipecoverspictures(),
        base_area = 10,
        base_level = -1,
        volume = 500,
        --pipe_connections = {{ type = "input", position = {-1, -5} }}
        --pipe_connections = {{ flow_direction="input", direction = defines.direction.north, position = {-1, -5} }},
        pipe_connections =
        {
          {
            flow_direction="input",
            direction = defines.direction.south,
            position = {-1, 2}
          }
        }
        --secondary_draw_orders = { north = -1 }
      },
      {
        production_type = "input",
        pipe_picture = assembler3pipepictures(),
        pipe_covers = pipecoverspictures(),
        base_area = 10,
        base_level = -1,
        volume = 500,
        --pipe_connections = {{ type = "input", position = {1, -5} }}
        --pipe_connections = {{ flow_direction="input", direction = defines.direction.south, position = {0, 1} }},
        pipe_connections =
        {
          {
            flow_direction="input",
            direction = defines.direction.south,
            position = {1, 2}
          }
        }
        --secondary_draw_orders = { north = -1 }
      },
    }, 

    fluid_boxes_off_when_no_fluid_recipe = true,
    collision_box = {{-4.2, -4.2}, {4.2, 4.2}},
    selection_box = {{-4.5, -4.5}, {4.5, 4.5}},

    animation = {
      filename = ENTITYPATH .. "Bio_Farm_Idle.png",
      priority = "high",
      width = 348,
      height = 288,
      shift = {0.96, 0},
      frame_count = 1,
    },

    working_visualisations = {
      animation = {
        filename = ENTITYPATH .. "Bio_Farm_Working.png",
        priority = "high",
        width = 348,
        height = 288,
        shift = {0.96, 0},
        frame_count = 1,
      },
    },
    crafting_categories = {"biofarm-mod-farm"},
    crafting_speed = 1,
    energy_source = {
      type = "electric",
      usage_priority = "primary-input",
      drain = "50kW",
      emissions_per_minute = { pollution = -9 }, -- the "-" means it Absorbs pollution.
    },
    energy_usage = "100kW",
    ingredient_count = 3,
    open_sound = { filename = "__base__/sound/machine-open.ogg", volume = 0.85 },
    close_sound = { filename = "__base__/sound/machine-close.ogg", volume = 0.75 },
    --vehicle_impact_sound = sounds.generic_impact,
    module_specification = {
      module_slots = 3
    },
    allowed_effects = {"consumption", "speed", "productivity", "pollution"},
  },


})

 

data:extend({
 
  ------ Greenhouse
  {
    type = "assembling-machine",
    name = "bi-bio-greenhouse",
    icon = ICONPATH .. "bio_greenhouse.png",
    icon_size = 64,
    icons = {
      {
        icon = ICONPATH .. "bio_greenhouse.png",
        icon_size = 64,
      }
    },
    flags = {"placeable-neutral", "placeable-player", "player-creation"},
    minable = {hardness = 0.2, mining_time = 0.25, result = "bi-bio-greenhouse"},
    collision_box = {{-1.2, -1.2}, {1.2, 1.2}},
    selection_box = {{-1.5, -1.5}, {1.5, 1.5}},
    max_health = 250,
    corpse = "medium-remnants",
    dying_explosion = "medium-explosion",
    crafting_categories = {"biofarm-mod-greenhouse"},
    crafting_speed = 1,
    energy_source = {
      type = "electric",
      usage_priority = "primary-input",
      drain = "15kW",
      emissions_per_minute = { pollution = -6 }, -- the "-" means it Absorbs pollution.
    },
    energy_usage = "50kW",
    ingredient_count = 3,
    resistances = {
      {
        type = "fire",
        percent = 70
      }
    },
    fluid_boxes = {
      {
        production_type = "input",
        pipe_picture = assembler3pipepictures(),
        pipe_covers = pipecoverspictures(),
        base_area = 10,
        base_level = -1,
        volume = 250,
        --pipe_connections = {{ type = "input", position = {0, -2} }}
        pipe_connections = {{ flow_direction="input", direction = defines.direction.north, position = {0, -1} }},
      },
    },


    module_specification = {
      module_slots = 2
    },
    allowed_effects = {"consumption", "speed", "productivity", "pollution"},
    animation = {
      filename = ENTITYPATH .. "bio_greenhouse_off.png",
      width = 113,
      height = 93,
      frame_count = 1,
      scale = 1,
      shift = {0.3, 0}
    },
    working_visualisations = {
      {
        light = {intensity = 1, size = 6},
        animation = {
          filename = ENTITYPATH .. "bio_greenhouse_on.png",
          width = 113,
          height = 93,
          frame_count = 1,
          scale = 1,
          shift = {0.3, 0}
        }
      }
    },
    open_sound = { filename = "__base__/sound/machine-open.ogg", volume = 0.85 },
    close_sound = { filename = "__base__/sound/machine-close.ogg", volume = 0.75 },
    --vehicle_impact_sound = sounds.generic_impact,
  },

  -- COKERY
  {
    type = "assembling-machine",
    name = "bi-cokery",
    icon = ICONPATH .. "cokery.png",
    icon_size = 64,
    icons = {
      {
        icon = ICONPATH .. "cokery.png",
        icon_size = 64,
      }
    },
    -- This is necessary for "Space Exploration" (if not true, the entity can only be
    -- placed on Nauvis)!
    se_allow_in_space = true,
    flags = {"placeable-neutral", "placeable-player", "player-creation"},
    order = "a[cokery]",
    minable = {hardness = 0.2, mining_time = 0.5, result = "bi-cokery"},
    max_health = 200,
    corpse = "medium-remnants",
    resistances = {{type = "fire", percent = 95}},
    collision_box = {{-1.2, -1.2}, {1.2, 1.2}},
    selection_box = {{-1.5, -1.5}, {1.5, 1.5}},
    module_specification = {
      module_slots = 2
    },
    allowed_effects = {"consumption", "speed", "pollution"},
    animation = {
      filename = "__Bio_Industries__/graphics/entities/cokery/cokery_sheet.png",
      frame_count = 28,
      line_length = 7,
      width = 256,
      height = 256,
      scale = 0.5,
      shift = {0.5, -0.5},
      animation_speed = 0.1
    },
    crafting_categories = {"biofarm-mod-smelting"},
    energy_source = {
      type = "electric",
      input_priority = "secondary",
      usage_priority = "secondary-input",
      emissions_per_minute = { pollution = 2.5 }, -- a "-" would means it Absorbs pollution.
    },
    energy_usage = "180kW",
    crafting_speed = 2,
    ingredient_count = 1
  },

  -- STONECRUSHER
  {
    type = "furnace",
    name = "bi-stone-crusher",
    icon = ICONPATH .. "stone_crusher.png",
    icon_size = 64,
    -- This is necessary for "Space Exploration" (if not true, the entity can only be
    -- placed on Nauvis)!
    se_allow_in_space = true,
    flags = {"placeable-neutral", "player-creation"},
    minable = {hardness = 0.2, mining_time = 0.5, result = "bi-stone-crusher"},
    max_health = 100,
    corpse = "medium-remnants",
    module_slots = 1,
    resistances = {{type = "fire", percent = 70}},
    working_sound = {
      sound = {
        filename = "__base__/sound/assembling-machine-t1-1.ogg",
        volume = 0.7
      },
      apparent_volume = 1.5
    },
    collision_box = {{-0.8, -0.8}, {0.8, 0.8}},
    selection_box = {{-1.0, -1.0}, {1.0, 1.0}},
    animation = {
      filename = "__Bio_Industries__/graphics/entities/stone-crusher/stone_crusher_anim.png",
      priority = "high",
      width = 65,
      height = 78,
      frame_count = 11,
      animation_speed = 0.5,
      shift = {0.0, -0.1}
    },
    working_visualisations = {
      filename = "__Bio_Industries__/graphics/entities/stone-crusher/stone-crusher-anim.png",
      priority = "high",
      width = 65,
      height = 78,
      frame_count = 11,
      animation_speed = 0.18 / 2.5,
      shift = {0.0, -0.1}
    },
    crafting_categories = {"biofarm-mod-crushing"},
    result_inventory_size = 1,
    source_inventory_size = 1,
    crafting_speed = 1,
    energy_source = {
      type = "electric",
      usage_priority = "secondary-input",      
      emissions_per_minute = { pollution = 0.25 }, -- a "-" would means it Absorbs pollution.
    },
    energy_usage = "50kW",
    module_specification = {
      module_slots = 2
    },
    allowed_effects = {"consumption", "speed", "pollution"},
  },

  --- Seed Bomb Projectile - 1
  {
    type = "projectile",
    name = "seed-bomb-projectile-1",
    flags = {"not-on-map"},
    acceleration = 0.005,
    action = {
      type = "direct",
      action_delivery = {
        type = "instant",
        target_effects = {
          {
            type = "nested-result",
            action = {
              type = "area",
              target_entities = false,
              repeat_count = 600,
              radius = 24,
              action_delivery = {
                type = "projectile",
                projectile = "seed-bomb-wave-1",
                starting_speed = 0.5
              }
            }
          }
        }
      }
    },
    light = {intensity = 0.8, size = 15},
    animation = {
      filename = "__Bio_Industries__/graphics/entities/biofarm/rocket/rocket.png",
      frame_count = 8,
      line_length = 8,
      width = 9,
      height = 35,
      shift = {0, 0},
      priority = "high"
    },
    shadow = {
	  filename = "__Bio_Industries__/graphics/entities/biofarm/rocket/rocket-shadow.png",
      frame_count = 1,
      width = 7,
      height = 24,
      priority = "high",
      shift = {0, 0}
    },
    smoke = {
      {
        name = "smoke-fast",
        deviation = {0.15, 0.15},
        frequency = 1,
        position = {0, -1},
        slow_down_factor = 1,
        starting_frame = 3,
        starting_frame_deviation = 5,
        starting_frame_speed = 0,
        starting_frame_speed_deviation = 5
      }
    }
  },

  --- Seed Bomb Projectile - 2
  {
    type = "projectile",
    name = "seed-bomb-projectile-2",
    flags = {"not-on-map"},
    acceleration = 0.005,
    action = {
      type = "direct",
      action_delivery = {
        type = "instant",
        target_effects = {
          {
            type = "nested-result",
            action = {
              type = "area",
              target_entities = false,
              repeat_count = 800,
              radius = 27,
              action_delivery = {
                type = "projectile",
                projectile = "seed-bomb-wave-2",
                starting_speed = 0.5
              }
            }
          }
        }
      }
    },
    light = {intensity = 0.8, size = 15},
    animation = {
      filename = "__Bio_Industries__/graphics/entities/biofarm/rocket/rocket.png",
      frame_count = 8,
      line_length = 8,
      width = 9,
      height = 35,
      shift = {0, 0},
      priority = "high"
    },
    shadow = {
      filename = "__Bio_Industries__/graphics/entities/biofarm/rocket/rocket-shadow.png",
      frame_count = 1,
      width = 7,
      height = 24,
      priority = "high",
      shift = {0, 0}
    },
    smoke = {
      {
         name = "smoke-fast",
         deviation = {0.15, 0.15},
         frequency = 1,
         position = {0, -1},
         slow_down_factor = 1,
         starting_frame = 3,
         starting_frame_deviation = 5,
         starting_frame_speed = 0,
         starting_frame_speed_deviation = 5
      }
    }
  },

  --- Seed Bomb Projectile - 3
  {
    type = "projectile",
    name = "seed-bomb-projectile-3",
    flags = {"not-on-map"},
    acceleration = 0.005,
    action = {
      type = "direct",
      action_delivery = {
        type = "instant",
        target_effects = {
          {
            type = "nested-result",
            action = {
              type = "area",
              target_entities = false,
              repeat_count = 1000,
              radius = 30,
              action_delivery = {
                type = "projectile",
                projectile = "seed-bomb-wave-3",
                starting_speed = 0.5
              }
            }
          }
        }
      }
    },
    light = {intensity = 0.8, size = 15},
    animation = {
      filename = "__Bio_Industries__/graphics/entities/biofarm/rocket/rocket.png",
      frame_count = 8,
      line_length = 8,
      width = 9,
      height = 35,
      shift = {0, 0},
      priority = "high"
    },
    shadow = {
      filename = "__Bio_Industries__/graphics/entities/biofarm/rocket/rocket-shadow.png",
      frame_count = 1,
      width = 7,
      height = 24,
      priority = "high",
      shift = {0, 0}
    },
    smoke = {
      {
        name = "smoke-fast",
        deviation = {0.15, 0.15},
        frequency = 1,
        position = {0, -1},
        slow_down_factor = 1,
        starting_frame = 3,
        starting_frame_deviation = 5,
        starting_frame_speed = 0,
        starting_frame_speed_deviation = 5
      }
    }
  },

  --- Seed Bomb Wave - 1
  {
    type = "projectile",
    name = "seed-bomb-wave-1",
    flags = {"not-on-map"},
    acceleration = 0,
    action = {
      {
        type = "direct",
        action_delivery = {
          type = "instant",
          target_effects = {
            {
              type = "create-entity",
              entity_name = "seedling",
              check_buildability = true,
              trigger_created_entity = true,
            }
          }
        }
      },
    },
    animation = {
      filename = "__core__/graphics/empty.png",
      frame_count = 1,
      width = 1,
      height = 1,
      priority = "high"
    },
    shadow = {
      filename = "__core__/graphics/empty.png",
      frame_count = 1,
      width = 1,
      height = 1,
      priority = "high"
    }
  },

  --- Seed Bomb Wave - 2
  {
    type = "projectile",
    name = "seed-bomb-wave-2",
    flags = {"not-on-map"},
    acceleration = 0,
    action = {
      {
        type = "direct",
        action_delivery = {
          type = "instant",
          target_effects = {
            {
              type = "create-entity",
              entity_name = "seedling-2",
              check_buildability = true,
              trigger_created_entity = true,
            }
          }
        }
      },
    },
    animation = {
      filename = "__core__/graphics/empty.png",
      frame_count = 1,
      width = 1,
      height = 1,
      priority = "high"
    },
    shadow = {
      filename = "__core__/graphics/empty.png",
      frame_count = 1,
      width = 1,
      height = 1,
      priority = "high"
    }
  },

  --- Seed Bomb Wave - 3
  {
    type = "projectile",
    name = "seed-bomb-wave-3",
    flags = {"not-on-map"},
    acceleration = 0,
    action = {
      {
        type = "direct",
        action_delivery = {
          type = "instant",
          target_effects = {
            {
              type = "create-entity",
              entity_name = "seedling-3",
              check_buildability = true,
              trigger_created_entity = true,
            },
          }
        }
      },
    },
    animation = {
      filename = "__core__/graphics/empty.png",
      frame_count = 1,
      width = 1,
      height = 1,
      priority = "high"
    },
    shadow = {
      filename = "__core__/graphics/empty.png",
      frame_count = 1,
      width = 1,
      height = 1,
      priority = "high"
    }
  },

    ---- Arboretum Area Overlay
  {
    type = "ammo-turret",
    name = "bi-arboretum-area",
    localised_name = {"entity-name.bi-arboretum"},
    localised_description = {"entity-description.bi-arboretum"},
    icon = ICONPATH .. "Arboretum_Icon.png",
    icon_size = 64,
    icons = {
      {
        icon = ICONPATH .. "Arboretum_Icon.png",
        icon_size = 64,
      }
    },
    flags = {"not-deconstructable", "not-on-map", "placeable-off-grid", "not-repairable"},
    open_sound = { filename = "__base__/sound/machine-open.ogg", volume = 0.85 },
    close_sound = { filename = "__base__/sound/machine-close.ogg", volume = 0.75 },
    max_health = 250,
    corpse = "big-remnants",
    dying_explosion = "medium-explosion",
    collision_box = {{-4.5, -4.5}, {4.5, 4.5}},
    selection_box = {{-4.5, -4.5}, {4.5, 4.5}},
    order = "x[bi]-a[bi-arboretum]",
    automated_ammo_count = 1,
    resistances = {},
    graphics_set = {},
    inventory_size = 1,
    attack_parameters = {
      type = "projectile",
      ammo_category = "bullet",
      cooldown = 2,
      range = 75,
      projectile_creation_distance = 0.1,
      action ={}
    },
    folding_speed = 0.08,
    folded_animation = (function()
        local res = util.table.deepcopy(inv_extension2)
        res.frame_count = 1
        res.line_length = 1
        return res
        end)(),

    folding_animation = (function()
        local res = util.table.deepcopy(inv_extension2)
        res.run_mode = "backward"
        return res
        end)(),

    call_for_help_radius = 1
  },


  --- Assembling-Machine Arboretum
  {
    type = "assembling-machine",
    name = "bi-arboretum",
    icon = ICONPATH .. "Arboretum_Icon.png",
    icon_size = 64,
    icons = {
      {
        icon = ICONPATH .. "Arboretum_Icon.png",
        icon_size = 64,
      }
    },
    flags = {"placeable-neutral", "placeable-player", "player-creation"},
    placeable_by = {item ="bi-arboretum-area", count = 1}, -- Fixes that entity couldn't be blueprinted
    minable = {hardness = 0.2, mining_time = 0.5, result = "bi-arboretum-area"},
    max_health = 250,
    corpse = "big-remnants",
    dying_explosion = "medium-explosion",
    resistances = {{type = "fire", percent = 70}},
    fluid_boxes = {
      {
        production_type = "input",
        pipe_picture = assembler3pipepictures(),
        pipe_covers = pipecoverspictures(),
        base_area = 10,
        base_level = -1,
        volume = 300,
        pipe_connections =
        {
          {
            flow_direction="input",
            direction = defines.direction.north,
            position = {-1, -1}
          }
        }
        --pipe_connections = {{ type = "input", position = {-1, -5} }}
      },
      {
        production_type = "input",
        pipe_picture = assembler3pipepictures(),
        pipe_covers = pipecoverspictures(),
        base_area = 10,
        base_level = -1,
        volume = 300,
        pipe_connections =
        {
          {
            flow_direction="input",
            direction = defines.direction.north,
            position = {1, -1}
          }
        }
        --pipe_connections = {{ type = "input", position = {1, -5} }}
      },
      
    },
    fluid_boxes_off_when_no_fluid_recipe = true,
    collision_box = {{-4.2, -4.2}, {4.2, 4.2}},
    selection_box = {{-4.5, -4.5}, {4.5, 4.5}},
    order = "x[bi]-a[bi-arboretum]",
    animation = {
      filename = ENTITYPATH .. "Bio_Farm_Idle_alt.png",
      priority = "low",
      width = 320,
      height = 320,
      frame_count = 1,
      shift = {0.75, 0},
    },

    working_visualisations = {
      animation = {
        filename = ENTITYPATH .. "Bio_Farm_Working_alt.png",
        priority = "low",
        width = 320,
        height = 320,
        frame_count = 1,
        shift = {0.75, 0},
      },
    },
    crafting_categories = {"bi-arboretum"},
    crafting_speed = 0.000000000001,
    energy_source = {
      type = "electric",
      usage_priority = "primary-input",
      emissions_per_minute = { pollution = -8 }, -- a "-" would means it Absorbs pollution.
    },
    energy_usage = "150kW",
    ingredient_count = 3,
    open_sound = { filename = "__base__/sound/machine-open.ogg", volume = 0.85 },
    close_sound = { filename = "__base__/sound/machine-close.ogg", volume = 0.75 },
   --vehicle_impact_sound = sounds.generic_impact,
    module_specification = {},
  },
})

